<?php
App::uses('EvSlideshowAppModel', 'EvSlideshow.Model');

/**
 * SlideshowSlide Model
 *
 */
class Slide extends EvSlideshowAppModel {

	public $imageSlots = array(
		'main' => array(
			'slots' => 1,
			'fields' => false
		)
	);

/**
 * Validation rules
 *
 * @var array
 */
	public $validate = array(
		'name' => array(
			'notEmpty' => array(
				'rule' => array('notBlank'),
				'message' => 'A name for the slide must be entered'
			),
		),
	);

	public $belongsTo = [
		'Slideshow' => [
			'className' => 'EvSlideshow.Slideshow'
		],
	];

/**
 * Constructor. Binds the model's database table to the object.
 *
 * @param bool|int|string|array $id Set this ID for this model on startup,
 * can also be an array of options, see above.
 * @param string $table Name of database table to use.
 * @param string $ds DataSource connection name.
 * @return void.
 */
	public function __construct($id = false, $table = null, $ds = null) {
		parent::__construct($id, $table, $ds);
		$this->_setImageSlots();
		$this->_setExtraValidation();
	}

	protected function _setImageSlots() {
		// bring in the config based image slot settings
		$configImageSlots = Configure::read('EvSlideshow.imageSlots');

		// assign, or merge, the config based image slots declaration
		if (! empty($configImageSlots)) {
			if (! empty($this->imageSlots)) {
				$this->imageSlots = array_merge(
					$this->imageSlots,
					$configImageSlots
				);
			} else {
				$this->imageSlots = $configImageSlots;
			}
		}
	}

	protected function _setExtraValidation() {
		$fields = Configure::read('EvSlideshow.fields');
		if (empty($fields['extra'])) {
			return;
		}

		foreach ($fields['extra'] as $name => $field) {
			if (empty($field['validate'])) {
				continue;
			}
			$this->validate["extra_$name"] = $field['validate'];
		}
	}

/**
 * Get active slides.
 *
 * @param array $params (optional) find parameters
 * @return array
 */
	public function getSlides($params = array()) {
		$defaultParams = array(
			'order' => array('sequence' => 'ASC'),
			'contain' => array(
				'Image'
			),
			'conditions' => array(
				$this->escapeField('is_active') => true
			)
		);

		$params = array_merge_recursive($defaultParams, $params);

		$data = $this->find('all', $params);

		return $data;
	}

	public function beforeSave($options = array()) {
		if (!parent::beforeSave($options)) {
			return false;
		}

		$extra = array();
		$extraFound = false;

		foreach ($this->data[$this->alias] as $field => $value) {

			if (substr($field, 0, 6) == 'extra_') {

				$extraFound = true;
				$extra[substr($field, 6)] = $value;
				unset($this->data[$this->alias][$field]);

			}

		}

		if ($extraFound) {
			$this->data[$this->alias]['extra'] = serialize($extra);
		}

		// Insert sequence number if creating a new slide.
		if (empty($this->id) && empty($this->data[$this->alias]['sequence'])) {
			$this->data[$this->alias]['sequence'] = $this->getNextSequenceNo(array(
					'conditions' => array(
						'slideshow_id' => $this->data[$this->alias]['slideshow_id'],
					)
				)
			);
		}

		return true;
	}

	public function afterFind($results, $primary = false) {
		foreach ($results as $k => $slide) {

			if (isset($slide[$this->alias]['extra'])) {

				$extra = unserialize($slide[$this->alias]['extra']);

				if (is_array($extra)) {

					foreach ($extra as $key => $value) {

						$results[$k][$this->alias]['extra_' . $key] = $value;
					}
				}
				unset($results[$k][$this->alias]['extra']);

			}

		}

		return $results;
	}

}
